/* Copyright 2020-2025, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <cstdint>
#include <vector>
#include <memory>
#include <string_view>
#include <string>
#include <map>

namespace mc::psx {
template <bool C> class BasicBlock;
} //namespace mc::psx

namespace duck {
struct BlockEntry {
	BlockEntry (
		std::vector<mc::psx::BasicBlock<false> >&& blocks,
		std::unique_ptr<uint8_t[]>&& data
	);

	std::string_view identifier() const;
	std::string title() const;

	std::vector<mc::psx::BasicBlock<false> > blocks;
	std::unique_ptr<uint8_t[]> data;
};

class BlockDb {
public:
	BlockDb();
	~BlockDb() noexcept;

	template <bool Const>
	unsigned int add (const std::vector<mc::psx::BasicBlock<Const> >& blocks);

private:
	std::map<unsigned int, BlockEntry> m_saves;
	std::size_t m_next_id{0};
};
} //namespace duck
