/* Copyright 2020-2025, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <QPoint>
#include <functional>
#include <utility>

class QMouseEvent;
class QPixmap;
class QWidget;
class QDragEnterEvent;
class QDropEvent;

namespace duck::widget::detail {
class BlockDragAndDrop {
public:
	typedef std::function<std::pair<int,QPoint>(QPoint)> PointToIndexFunc;

	BlockDragAndDrop (
		QWidget* parent,
		PointToIndexFunc&& func,
		std::function<std::size_t()>&& get_icon_count,
		std::function<const QPixmap&(std::size_t)>&& get_pixmap
	);

	void mouse_press_event (const QMouseEvent* event);
	void mouse_release_event (const QMouseEvent* event);
	void mouse_move_event (const QMouseEvent* event);
	void drag_enter_event(QDragEnterEvent* event);
	void drop_event(QDropEvent* event);

private:
	PointToIndexFunc m_point_to_index;
	std::function<std::size_t()> m_get_icon_count;
	std::function<const QPixmap&(std::size_t)> m_get_pixmap;
	QWidget* m_parent;
	QPoint m_start_pos;
	QPoint m_hot_spot;
	int m_block_index;
	bool m_dragging {false};
};

} //namespace duck::widget::detail
